/**
 * @file getting_started.cpp
 *
 * @author FTDI
 * @date 2014-07-01
 *
 * Copyright © 2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * The sample source code is provided as an example and is neither guaranteed
 * nor supported by FTDI.
 *
 * Revision History:
 * 1.0 - initial version
 */

//------------------------------------------------------------------------------
#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include <string>

//------------------------------------------------------------------------------
// include FTDI libraries
//
#include "ftd2xx.h"
#include "LibFT4222.h"


std::vector< FT_DEVICE_LIST_INFO_NODE > g_FT4222DevList;

//------------------------------------------------------------------------------
inline std::string DeviceFlagToString(DWORD flags)
{
    std::string msg;
    msg += (flags & 0x1)? "DEVICE_OPEN" : "DEVICE_CLOSED";
    msg += ", ";
    msg += (flags & 0x2)? "High-speed USB" : "Full-speed USB";
    return msg;
}

void ListFtUsbDevices()
{
    FT_STATUS ftStatus = 0;

    DWORD numOfDevices = 0;
    ftStatus = FT_CreateDeviceInfoList(&numOfDevices);

    for(DWORD iDev=0; iDev<numOfDevices; ++iDev)
    {
        FT_DEVICE_LIST_INFO_NODE devInfo;
        memset(&devInfo, 0, sizeof(devInfo));

        ftStatus = FT_GetDeviceInfoDetail(iDev, &devInfo.Flags, &devInfo.Type, &devInfo.ID, &devInfo.LocId,
                                        devInfo.SerialNumber,
                                        devInfo.Description,
                                        &devInfo.ftHandle);

        if (FT_OK == ftStatus)
        {
            printf("Dev %d:\n", iDev);
            printf("  Flags= 0x%x, (%s)\n", devInfo.Flags, DeviceFlagToString(devInfo.Flags).c_str());
            printf("  Type= 0x%x\n",        devInfo.Type);
            printf("  ID= 0x%x\n",          devInfo.ID);
            printf("  LocId= 0x%x\n",       devInfo.LocId);
            printf("  SerialNumber= %s\n",  devInfo.SerialNumber);
            printf("  Description= %s\n",   devInfo.Description);
            printf("  ftHandle= 0x%p\n",    devInfo.ftHandle);

            const std::string desc = devInfo.Description;
            if(desc == "FT4222" || desc == "FT4222 A")
            {
                g_FT4222DevList.push_back(devInfo);
            }
        }
    }
}

void GetLibInfo()
{
#if defined(_WIN64)
    printf("Platform: x64\n");
    const WCHAR LibFT4222Name[] = L"LibFT4222-64.dll";
#elif defined(_WIN32)
    printf("Platform: x86\n");
    const WCHAR LibFT4222Name[] = L"LibFT4222.dll";
#else
    printf("Platform: unknow\n");
#endif

    const WCHAR LibFTd2xxName[] = L"FTD2XX.dll";
    WCHAR path[MAX_PATH];
    HMODULE hModule;
    // get exe path
    if (GetModuleFileName(NULL, path, MAX_PATH)) {
        wprintf(L"Executable Path: %s\n", path);
    }
    // get libft4222 dll path
    hModule = GetModuleHandle(LibFT4222Name);
    if (hModule) {
        if (GetModuleFileName(hModule, path, MAX_PATH)) {
            wprintf(L"DLL Path: %s\n", path);
        }
        else {
            wprintf(L"Failed to get DLL path: %s\n", LibFT4222Name);
        }
    }
    else {
        wprintf(L"DLL not loaded: %s\n", LibFT4222Name);
    }
    // get ftd2xx dll path
    hModule = GetModuleHandle(LibFTd2xxName);
    if (hModule) {
        if (GetModuleFileName(hModule, path, MAX_PATH)) {
            wprintf(L"DLL Path: %s\n", path);
        }
        else {
            wprintf(L"Failed to get DLL path: %s\n", LibFTd2xxName);
        }
    }
    else {
        wprintf(L"DLL not loaded: %s\n", LibFTd2xxName);
    }

    // get version
    FT_HANDLE ftHandle = NULL;
    FT_STATUS ftStatus;
    LPDWORD lpdwVersion = new DWORD;
    FT4222_Version ft4222Version;
    ftStatus = FT_OpenEx((PVOID)(uintptr_t)g_FT4222DevList[0].LocId, FT_OPEN_BY_LOCATION, &ftHandle);
    if (FT_OK != ftStatus)
    {
        printf("Open a FT4222 device failed!\n");
        return;
    }
    ftStatus = FT_GetDriverVersion(ftHandle, lpdwVersion);
    if (FT_OK != ftStatus)
    {
        printf("Get LibFTD2XX Driver Version failed!\n");
        FT_Close(ftHandle);
        return;
    }
    printf("FTD2XX DriverVersion: %x\n", *lpdwVersion);
    ftStatus = FT_GetLibraryVersion(lpdwVersion);
    if (FT_OK != ftStatus)
    {
        printf("Get LibFTD2XX Library Version failed!\n");
        FT_Close(ftHandle);
        return;
    }
    printf("FTD2XX LibraryVersion: %x\n", *lpdwVersion);    
    ftStatus = FT4222_GetVersion(ftHandle, &ft4222Version);
    if (FT_OK != ftStatus)
    {
        printf("Get LibFT4222 Version failed!\n");
        FT_Close(ftHandle);
        return;
    }
    printf("FT4222 ChipVersion:%x\n", ft4222Version.chipVersion);
    printf("FT4222 DllVersion:%x\n", ft4222Version.dllVersion);
    FT_Close(ftHandle);
}

//------------------------------------------------------------------------------
// main
//------------------------------------------------------------------------------
int main(int argc, char const *argv[])
{

    ListFtUsbDevices();
    GetLibInfo();

    if(g_FT4222DevList.empty()) {
        printf("No FT4222 device is found!\n");
        return 0;
    }

    FT_HANDLE ftHandle = NULL;

    FT_STATUS ftStatus;
    ftStatus = FT_OpenEx((PVOID)(uintptr_t)g_FT4222DevList[0].LocId, FT_OPEN_BY_LOCATION, &ftHandle);
    if (FT_OK != ftStatus)
    {
        printf("Open a FT4222 device failed!\n");
        return 0;
    }


    printf("\n\n");
    printf("Init FT4222 as SPI master\n");
    ftStatus = FT4222_SPIMaster_Init(ftHandle, SPI_IO_SINGLE, CLK_DIV_4, CLK_IDLE_LOW, CLK_LEADING, 0x01);
    if (FT_OK != ftStatus)
    {
        printf("Init FT4222 as SPI master device failed!\n");
        return 0;
    }

    // TODO:
    //    Start to work as SPI master, and read/write data to a SPI slave
    //    FT4222_SPIMaster_SingleWrite
    //    FT4222_SPIMaster_SingleRead
    //    FT4222_SPIMaster_SingleReadWrite

    printf("TODO ...\n");
    printf("\n");


    printf("UnInitialize FT4222\n");
    FT4222_UnInitialize(ftHandle);

    printf("Close FT device\n");
    FT_Close(ftHandle);
    return 0;
}
